---------------------------------------------------------------
--                                                           
--  ROBOT INTERPRETER
--  By: Martin Carlisle and Keith Shomper     
--                                                           
--  Controls
--  Description : Handle actions on controls in main window 
--                       
--  By: Martin Carlisle
--                                         
-- ROBOT INTERPRETER is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- ROBOT INTERPRETER is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Simulation_Gui;
with RobotADT;
with RoomADT;
with State;
with Mcc.Gui.Widget.Button.Picture;
with Main_Window;
with Mcc.Gui.Event_Scheduler;
with Mcc.Gui.Widget.Scale;
with ada.text_io;
with Mcc.Common_Dialogs;
package body Controls is
   Speed_Constant_Part : constant := 50;
   Speed_Slope_Part    : constant := 30;
   
   procedure Run_Timer_Callback;
   
   Running : Boolean := False;

   Wavy_Lines_X, Wavy_Lines_Y : Natural;
   Am_Waving : Boolean := False;
   
   procedure Unwavy_Lines is
      Scale_Value : Integer;
   begin
      Am_Waving := False;
      Simulation_Gui.Set_Image(
         Simulation_Gui.Room(Wavy_Lines_X,Wavy_Lines_Y),
         Simulation_Gui.Blank_Grid);  
      if Running and not RobotADT.End_Of_Program(State.Robot) then
         Scale_Value := Mcc.Gui.Widget.Scale.Get(
            Main_Window.Speed_Scale);
         Mcc.Gui.Event_Scheduler.Schedule_Event(
            Number_Of_Milliseconds => 
               Speed_Constant_Part+(100-Scale_Value)*Speed_Slope_Part,
            Event_Callback         => Run_Timer_Callback'Access);
      end if;
   end Unwavy_Lines;

   procedure Wavy_Lines(X,Y : in Natural) is
      Scale_Value : Integer;
      I,J : Natural;
   begin
      Am_Waving := True;
      Wavy_Lines_X := X;
      Wavy_Lines_Y := Simulation_Gui.Room'Last(2)-Y;
      RobotADT.Get_Location(State.Robot,I,J);
      if X=I then
         Simulation_Gui.Set_Image(
            Simulation_Gui.Room(Wavy_Lines_X,Wavy_Lines_Y),
            Simulation_Gui.Wavy_NS);  
      else
         Simulation_Gui.Set_Image(
            Simulation_Gui.Room(Wavy_Lines_X,Wavy_Lines_Y),
            Simulation_Gui.Wavy_EW);  
      end if;
      Scale_Value := Mcc.Gui.Widget.Scale.Get(
         Main_Window.Speed_Scale);
      Mcc.Gui.Event_Scheduler.Schedule_Event(
         Number_Of_Milliseconds => 
            Speed_Constant_Part/2+(100-Scale_Value)*Speed_Slope_Part/2,
         Event_Callback         => Unwavy_Lines'access);
   end Wavy_Lines;
   
   procedure Do_Next is
   begin
      if Am_Waving then
         return;
      end if;
      
      if not RobotADT.End_Of_Program(State.Robot) then
         Simulation_Gui.Erase_Robot;
         begin
            RobotADT.Fetch_And_Execute(State.Robot,Wavy_Lines'access);
         exception when RobotADT.Memory_Error =>
            Mcc.Common_Dialogs.Ok_Box("Program executed past end of memory");
            Running := False;
         end;
         Simulation_Gui.Draw_Robot;
      end if;
   end Do_Next;

   procedure Run_Timer_Callback is
      Scale_Value : Integer;
   begin
      if Running then
         Do_Next;
         if not RobotADT.End_Of_Program(State.Robot) then
            Scale_Value := Mcc.Gui.Widget.Scale.Get(
               Main_Window.Speed_Scale);
            if not Am_Waving then
               Mcc.Gui.Event_Scheduler.Schedule_Event(
                  Number_Of_Milliseconds => 
                     Speed_Constant_Part+(100-Scale_Value)*Speed_Slope_Part,
                  Event_Callback         => Run_Timer_Callback'Access);
            end if;
         else
            Running := False;
         end if;
      end if;
   end Run_Timer_Callback;

   -------------
   -- Do_Next --
   -------------

   procedure Do_Next (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Do_Next;
   end Do_Next;

   -----------------
   -- Reset_Robot --
   -----------------

   procedure Reset_Robot (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Stop(Obj);
      Simulation_Gui.Erase_Robot;
      RobotADT.set_PC (State.Robot,0);
      RobotADT.set_arms (State.Robot,RobotADT.down);
      RobotADT.set_light (State.Robot,RobotADT.off);
      Simulation_Gui.Draw_Robot;
      Mcc.Gui.Widget.Button.Picture.Set_Image(
         Main_Window.Toggle_Arms,
         Simulation_Gui.North_Off_Down);  
      Mcc.Gui.Widget.Button.Picture.Set_Image(
         Main_Window.Toggle_Light,
         Simulation_Gui.Light_Off);  
   end Reset_Robot;

   ----------------
   -- Robot_Down --
   ----------------

   procedure Robot_Down (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Simulation_Gui.Erase_Robot;
      begin
         RobotADT.set_direction(State.Robot,RobotADT.south);
      exception when RobotADT.Instruction_Error =>
         Mcc.Gui.Bell;
      end;
      Simulation_Gui.Draw_Robot;
   end Robot_Down;

   ----------------
   -- Robot_Left --
   ----------------

   procedure Robot_Left (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Simulation_Gui.Erase_Robot;
      begin
         RobotADT.set_direction(State.Robot,RobotADT.west);
      exception when RobotADT.Instruction_Error =>
         Mcc.Gui.Bell;
      end;
      Simulation_Gui.Draw_Robot;
   end Robot_Left;

   -----------------
   -- Robot_Right --
   -----------------

   procedure Robot_Right (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Simulation_Gui.Erase_Robot;
      begin
         RobotADT.set_direction(State.Robot,RobotADT.east);
      exception when RobotADT.Instruction_Error =>
         Mcc.Gui.Bell;
      end;
      Simulation_Gui.Draw_Robot;
   end Robot_Right;

   --------------
   -- Robot_Up --
   --------------

   procedure Robot_Up (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Simulation_Gui.Erase_Robot;
      begin
         RobotADT.set_direction(State.Robot,RobotADT.north);
      exception when RobotADT.Instruction_Error =>
         Mcc.Gui.Bell;
      end;
      Simulation_Gui.Draw_Robot;
   end Robot_Up;

   --------------------
   -- Run_Until_Stop --
   --------------------

   procedure Run_Until_Stop
     (Obj : in out Mcc.Gui.Widget.Button.Button'Class)
   is
   begin
      Running := True;
      Run_Timer_Callback;
   end Run_Until_Stop;

   ----------
   -- Stop --
   ----------
   procedure Stop is
   begin
      Running := False;
   end Stop;
   procedure Stop (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Stop;
   end Stop;

   ------------------
   -- Toggle_Light --
   ------------------

   procedure Toggle_Light (Obj : in out Mcc.Gui.Widget.Button.Button'Class)
   is
      use type RobotADT.Light_Type;
      light : RobotADT.Light_Type;
   begin
      RobotADT.get_light(State.Robot,light);
      Simulation_Gui.Erase_Robot;
      if light=RobotADT.on then
         RobotADT.set_light(State.Robot,RobotADT.off);
         Mcc.Gui.Widget.Button.Picture.Set_Image(
            Main_Window.Toggle_Light,
            Simulation_Gui.Light_Off);  
      else
         RobotADT.set_light(State.Robot,RobotADT.on);
         Mcc.Gui.Widget.Button.Picture.Set_Image(
            Main_Window.Toggle_Light,
            Simulation_Gui.Light_On);  
      end if;
      Simulation_Gui.Draw_Robot;
   end Toggle_Light;

   -----------------------
   -- Toggle_Robot_Arms --
   -----------------------

   procedure Toggle_Robot_Arms
     (Obj : in out Mcc.Gui.Widget.Button.Button'Class)
   is
      use type RobotADT.Arm_Type;
      arms : RobotADT.Arm_Type;
   begin
      RobotADT.get_arms(State.Robot,arms);
      Simulation_Gui.Erase_Robot;
      if arms=RobotADT.down then
         RobotADT.set_arms(State.Robot,RobotADT.up);
         Mcc.Gui.Widget.Button.Picture.Set_Image(
            Main_Window.Toggle_arms,
            Simulation_Gui.North_Off_Up);  
      else
         RobotADT.set_arms(State.Robot,RobotADT.down);
         Mcc.Gui.Widget.Button.Picture.Set_Image(
            Main_Window.Toggle_Arms,
            Simulation_Gui.North_Off_Down);  
      end if;
      Simulation_Gui.Draw_Robot;
   end Toggle_Robot_Arms;

end Controls;

