with cs110;
use  cs110;

with Ada.Text_IO;
with Ada.Integer_Text_IO;

package body robot is

   package Inst_IO is new Ada.Text_IO.Enumeration_IO (Enum => instruction_type);
   
   ----------
   -- draw --
   ----------

   procedure draw (R : in robot_type) is
      X1, X2, X3, X4 : Integer;
      Y1, Y2, Y3, Y4 : Integer;
      X, Y           : Integer;
   begin
      X := Room.Tile_to_Screen_X(X => R.X);
      Y := Room.Tile_to_Screen_Y(Y => R.Y);

      if R.arms = up then
         case R.direction is
            when Common.north =>
               -- left arm
               X1 := X - 1;             Y1 := Y;
               X2 := X - 1 - Arm_Width; Y2 := Y + Arm_Length;

               -- right arm
               X3 := X + 1;              Y3 := Y;
               X4 := X + 1 + Arm_Width; Y4 := Y + Arm_Length;
            when Common.south =>
               -- left arm
               X1 := X + 1;             Y1 := Y;
               X2 := X + 1 + Arm_Width; Y2 := Y - Arm_Length;

               -- right arm
               X3 := X - 1;             Y3 := Y;
               X4 := X - 1 - Arm_Width; Y4 := Y - Arm_Length;
            when Common.east  =>
               -- left arm
               X1 := X;              Y1 := Y + 1;
               X2 := X + Arm_Length; Y2 := Y + 1 + Arm_Width;

               -- right arm
               X3 := X;              Y3 := Y - 1;
               X4 := X + Arm_Length; Y4 := Y - 1 - Arm_Width;
            when Common.west  =>
               -- left arm
               X1 := X;              Y1 := Y - 1;
               X2 := X - Arm_Length; Y2 := Y - 1 - Arm_Width;

               -- right arm
               X3 := X;              Y3 := Y + 1;
               X4 := X - Arm_Length; Y4 := Y + 1 + Arm_Width;
         end case;
         Adagraph.Draw_Box(X1 => X1, Y1 => Y1, X2 => X2, Y2 => Y2, Hue => Arm_Color, Filled => True);
         Adagraph.Draw_Box(X1 => X3, Y1 => Y3, X2 => X4, Y2 => Y4, Hue => Arm_Color, Filled => True);
      end if;

      Adagraph.Draw_Circle (X => X, Y => Y, Radius => Body_Radius, Hue => Body_Color, Filled => True);
      if R.light = on then
         Adagraph.Draw_Circle (X => X, Y => Y, Radius => Light_Radius, Hue => Light_Color, Filled => True);
      end if;
   end draw;

   -------------
   -- execute --
   -------------

   procedure execute (R: in out Robot_Type) is
      instruction : instruction_type;
   begin
         instruction := R.Memory(R.PC).Instruction;
         case instruction is
            when stepi  =>
               step (R => R);
            when turni  =>
               turn (R => R);
            when raisei =>
               raise_arms (R => R);
            when loweri =>
               lower_arms (R => R);
            when sensei =>
               sense (R => R);
            when gotoi  =>
               jump (R => R);
            when lighti =>
               light (R);
            when stopi  =>
               Put (Item => "Stop instruction executed.");
               New_Line;
            -- this last case should never be executed because procedure load should
            -- have flagged any errors in the input program
            when others =>
               raise instruction_error;
         end case;
   end execute;

   -------------
   --  Fetch  --
   -------------

   procedure fetch (R: in out robot_type) is
   begin
      if not R.jump then
         R.pc := R.pc + 1;
      end if;
      R.Jump := False;
   end fetch;

   --------------------
   --  End_of_Progam --
   --------------------

   function  end_of_program (R: robot_type) return Boolean is
   begin
      if R.memory(R.pc).instruction = stopi or (R.X < 1 or R.X > Room.Width) or (R.Y < 1 or R.Y > Room.Height) then
         return True;
      else
         return False;
      end if;
   end end_of_program;

   -------------
   --  Place  --
   -------------

   procedure place (R : in out robot_type; X : in Positive; Y : in Positive) is
   begin
      R.X      := X;
      R.Y      := Y;
   end place;

   ---------------
   -- Location  --
   ---------------

   procedure location (R : in robot_type; X : out Positive; Y : out Positive) is
   begin
      X      := R.X;
      Y      := R.Y;
   end location;

   ----------
   -- load --
   ----------

   procedure load (R : in out robot_type) is
      Filename    : String (1..80);
      File        : Ada.Text_IO.File_Type;
      Length      : Natural;
      I           : Integer := 0;
      operand     : Natural;
   begin
      Put (Item => "Please enter input filename:  ");
      Ada.Text_IO.Get_Line (Item => Filename, Last => Length);
      
      Ada.Text_IO.Open(Name => Filename(1..Length), Mode => Ada.Text_IO.In_File, File => File);
      
      while not Ada.Text_IO.End_Of_File (File) loop
         begin -- exeception handler for file input
         
            -- Get a robot instruction 
            Inst_IO.Get(File => File, Item => R.Memory(I).instruction);
               
            -- Only the gotoi instruction has an operand   
            if R.Memory(I).instruction = gotoi then
               Ada.Integer_Text_IO.Get(File => File, Item => operand);
               R.Memory(I).Operand := operand_type(operand);
            end if;

            -- Skip the remainder (if any) of the record
            Ada.Text_IO.Skip_Line(File => File);

            I := I + 1;
         exception -- handler for file input
            when Ada.Text_IO.End_Error => 
               null;
            when Ada.Text_IO.Data_Error =>
               New_Line;
               Put (Item => "Data error in record:");
               Ada.Text_IO.Set_Col(30);
               Put (Item => I);
               Put (Item => ".  Discarding this record.");
               New_Line;
               Ada.Text_IO.Skip_Line (File => File);
               delay 0.5;
            when Constraint_Error =>
               New_Line;
               Put (Item => "Constraint error in record:");
               Ada.Text_IO.Set_Col(30);
               Put (Item => I);
               Put (Item => ".  Discarding this record.");
               New_Line;
               Ada.Text_IO.Skip_Line (File => File);
               delay 0.5;
         end; -- exception handler for file input
      end loop;
      
      Ada.Text_IO.Close (File => File);
   end load;

   -------------
   --  Gotoi  --
   -------------

   procedure jump (R : in out robot_type) is
   begin
      if R.light = off then
         R.pc := R.Memory(R.pc).operand;
         R.jump := True;
      end if;
   end jump;
 
   -----------
   -- light --
   -----------

   procedure light (R : in out robot_type) is
   begin
      R.light := off;
   end light;

   ----------------
   -- lower_arms --
   ----------------

   procedure lower_arms (R : in out robot_type) is
   begin
      if R.light = off then
         R.arms := down;
      end if;
   end lower_arms;

   ----------------
   -- raise_arms --
   ----------------

   procedure raise_arms (R : in out robot_type) is
      door : Boolean;
   begin
      if R.light = off then
         door := Room.Is_Door (R.X, R.Y, R.direction);
         case R.direction is
            when Common.North =>
               if R.Y = Room.Height and not door then
                  R.light := on;
               else
                  R.arms := up;
               end if;
            when Common.South =>
               if R.Y = 1 and not door then
                  R.light := on;
               else
                  R.arms := up;
               end if;
            when Common.East  =>
               if R.X = Room.Width and not door then
                  R.light := on;
               else
                  R.arms := up;
               end if;
            when Common.West  =>
               if R.X = 1 and not door then
                  R.light := on;
               else
                  R.arms := up;
               end if;
         end case;
      end if;
   end raise_arms;

   -----------
   -- sense --
   -----------

   procedure sense (R : in out robot_type) is
      door : Boolean;
   begin
      if R.arms = up and R.light = off then
         case R.direction is
            when Common.north =>
               door := Room.Is_Door(R.X, R.Y+1, R.direction);
               if R.Y + 1 = Room.Height and not door then
                  R.light := on;
               end if;
            when Common.south =>
               door := Room.Is_Door(R.X, R.Y-1, R.direction);
               if R.Y - 1 = 1 and not door then
                  R.light := on;
               end if;
            when Common.east  =>
               door := Room.Is_Door(R.X+1, R.Y, R.direction);
               if R.X + 1 = Room.Width and not door then
                  R.light := on;
               end if;
            when Common.west  =>
               door := Room.Is_Door(R.X-1, R.Y, R.direction);
               if R.X - 1 = 1 and not door then
                  R.light := on;
               end if;
         end case;
      end if;
   end sense;

   ----------
   -- step --
   ----------

   procedure step (R : in out robot_type) is
      door : Boolean;
   begin
      case R.direction is
         when Common.north =>
            door := Room.Is_Door (R.X, Room.Height, R.direction);
            if not ((R.Y = Room.Height and not door) or (R.Y + 1 = Room.Height and R.arms = up and not door)) then
               R.Y := R.Y + 1;
            end if;
         when Common.south =>
            door := Room.Is_Door (R.X, 1, R.direction);
            if not ((R.Y = 1 and not door) or (R.Y - 1 = 1 and R.arms = up and not door)) then
               R.Y := R.Y - 1;
            end if;
         when Common.east  =>
            door := Room.Is_Door (Room.Width, R.Y, R.direction);
            if not ((R.X = Room.Width and not door) or (R.X + 1 = Room.Width and R.arms = up and not door)) then
               R.X := R.X + 1;
            end if;
         when Common.west  =>
            door := Room.Is_Door (1, R.Y, R.direction);
            if not ((R.X = 1 and not door) or (R.X - 1 = 1 and R.arms = up and not door)) then
               R.X := R.X - 1;
            end if;
      end case;
   end step;

   ----------
   -- turn --
   ----------

   procedure turn (R : in out robot_type) is
      door : Boolean;
   begin
      if R.light = off then
         case R.direction is
         when Common.north =>
            door := Room.Is_Door (R.X, R.Y, Common.east);
            if not (R.X = Room.Width and R.arms = up and not door) then
               R.direction := Common.east;
            end if;
         when Common.south =>
            door := Room.Is_Door (R.X, R.Y, Common.west);
            if not (R.X = 1 and R.arms = up and not door) then
               R.direction := Common.west;
            end if;
         when Common.east  =>
            door := Room.Is_Door (R.X, R.Y, Common.south);
            if not (R.Y = 1 and R.arms = up and not door) then
               R.direction := Common.south;
            end if;
         when Common.west  =>
            door := Room.Is_Door (R.X, R.Y, Common.north);
            if not (R.Y = Room.Height and R.arms = up and not door) then
               R.direction := Common.north;
            end if;
         end case;
      end if;
   end turn;

end robot;

