package roomADT is
   type room_type     is private;
   type room_type_ptr is access all room_type; 
   type tile_type     is (open, door, wall);

   -- creates a room of the specified size, if num_doors is positive then create will randomly
   -- place num_doors doors about the room   
   procedure create    (R :    out room_type; size : in Positive; Num_Doors : in Natural := 0);

   -- creates a room of the specified height and width, if num_doors is positive then create will 
   -- randomly place num_doors doors about the room   
   procedure create    (R :    out room_type; height : in Positive; width : in Positive; 
      Num_Doors : in Natural := 0);

   -- downloads the room state from from the named file
   procedure load_room (R :    out room_type; filename : in String);

   -- writes the room state to the named file
   procedure store_room(R : in     room_type; filename : in String); 

   -- inquiry/modifier procedures for room tiles
   function  get_tile  (R : in     room_type; X : in Natural; Y : in Natural) return tile_type;
   procedure set_tile  (R : in out room_type; X : in Natural; Y : in Natural; tile : in  tile_type);

   -- Returns true if the specified location is a door; otherwise returns false
   function  is_door (R : room_type; X : Natural; Y : Natural) return Boolean;
   
   -- inquiry function on the room state
   function  size    (R : room_type) return Natural;
   function  width   (R : room_type) return Positive;
   function  height  (R : room_type) return Positive;

   private
   
   max_size              : constant positive := 30;
   max_height            : constant positive := max_size;
   max_width             : constant positive := max_height;
   
   type boolean_string     is array (Integer range <>) of Boolean;
   type tile_2D_array_type is array (0..max_width+1, 0..max_height+1) of tile_type;
   type room_type is record
      size      : Natural;
      height    : positive;
      width     : positive;
      tiles     : tile_2D_array_type := (others => (others => open));
   end record;

   -- creates a room of the specified height and width and doors as specified around the perimeter
   -- by a true value (position 1 on the perimeter is above the top, left room tile and proceeds
   -- clockwise.  Note:  the boolean string's length should be equal to 2*height + 2*width
   procedure create    (R :    out room_type; height : in Positive; width : in Positive; 
      Doors : in boolean_string);

end roomADT;